import json
import sys
from base64 import b64decode

sys.path.insert(0, '/motadata/motadata/motadata python engine/')

from com.motadata.commons.winrmclient import TraceOrgWinRMUtil
from com.motadata.util.loggerutil import TraceOrgLogger

_logger = TraceOrgLogger('Windows Process')


def collect(context):
    try:

        result = {}

        if context and context.get('plugin-context'):

            plugin_context = context.get("plugin-context")

            winrm_client = get_winrm_client(plugin_context, context['metric-collect-timeout'])

            if winrm_client and winrm_client.connected:

                process_command = str(plugin_context.get('process-command')).replace("_@#_",":").replace("\\","").replace("\\\\","")

                plugin_context['process-command'] = process_command

                output = winrm_client.execute_command(
                    "Get-WmiObject win32_processor | Format-List  -Property numberoflogicalprocessors")

                if output is not None and len(output) > 0:
                    result['logical_processor'] = output

                if plugin_context.get('process') is not None and len(plugin_context.get('process')) > 0:
                    output = winrm_client.execute_command("Get-WmiObject Win32_Process | where-object {$_.name -like '*@@@*'} "
                                                          "| select name, commandline, processId".replace('@@@', plugin_context.get(
                        'process')))

                    process_id = 0
                    if output is not None and len(output) > 0:
                        counter_values = output.split("\r\n")

                        for index in range(len(counter_values)):
                            if 'commandline' in counter_values[index]:
                                command_line = counter_values[index].strip().replace("commandline :", "").replace(
                                    ",", " ").replace("\\", "").replace("\\\\", "").replace("\"", "").strip()
                                if command_line.lower() == plugin_context.get('process-command').lower():
                                    if 'processId' in counter_values[index + 1]:
                                        process_id = counter_values[index + 1].replace("processId   :", "").strip()
                                    elif 'processId' in counter_values[index + 2]:
                                        process_id = counter_values[index + 2].replace("processId   :", "").strip()
                                if int(process_id) > 0:
                                    break

                        result['process_id_details'] = output

                    process = "'\Process(@@@*)\ID Process','\Process(@@@*)\% Processor Time','\Process(" \
                              "@@@*)\Thread Count','\Process(@@@*)\Working Set','\Process(@@@*)\IO Data " \
                              "Bytes/sec','\Process(@@@*)\Virtual Bytes',".replace('@@@',
                                                                                   plugin_context.get('process').replace('.exe', ''))

                    output = winrm_client.execute_command(
                        "(Get-Counter @@@  -ErrorAction SilentlyContinue).countersamples | Format-List  -Property Path,Cookedvalue".replace(
                            "@@@", process[:len(process) - 1]))

                    if output is not None and len(output) > 0:
                        result['metrics'] = output

                    if process_id is not 0:
                        output = winrm_client.execute_command(
                            "New-TimeSpan -Start (get-process -id " + process_id + ").StartTime "
                                                                                   "| "
                                                                                   "Format-List "
                                                                                   "-Property "
                                                                                   "Days,Hours,"
                                                                                   "Minutes,"
                                                                                   "Seconds,"
                                                                                   "TotalMilliseconds")

                        if output is not None and len(output) > 0:
                            result['uptime_metrics'] = output

                winrm_client.destroy()

        return result
    except Exception:
        _logger.error()

    return None

def discover(winrm_client, context):
    try:
        result = {}

        if context.get('process') is not None and len(context.get('process')) > 0:
            output = winrm_client.execute_command("Get-WmiObject Win32_Process | where-object {$_.name -like '*@@@*'} "
                                                  "| select name, commandline, processId".replace('@@@', context.get(
                'process')))
        elif context.get('process-command') is not None and len(context.get('process-command')) > 0:
            output = winrm_client.execute_command("Get-WmiObject Win32_Process | where-object {$_.commandline -like "
                                                  "'*@@@*'} | select name, commandline,processId".replace('@@@',
                                                                                                          context.get(
                                                                                                              'process-command')))
        else:
            output = winrm_client.execute_command("Get-WmiObject Win32_Computersystem | select caption")
            if output is not None and len(output) > 0:
                output = winrm_client.execute_command('Get-WmiObject Win32_Process | select name, commandline')

        if output is not None and len(output) > 0:
            result['result'] = output
            return result

    except Exception as exception:
        _logger.error()

    return None


def get_winrm_client(context,timeout):
    try:
        port = 5985

        end_point = ''

        if context.get('port') is not None:
            port = int(context.get('port'))

        if port == 5985:
            end_point = "http://" + context.get('host') + ":5985/wsman"
        elif port == 5986:
            end_point = "https://" + context.get('host') + ":5986/wsman"

        context['endpoint'] = end_point

        winrm_client = TraceOrgWinRMUtil(context['endpoint'],
                                         context['username'],
                                         context['password'],
                                         timeout)

        winrm_client.init()

        return winrm_client

    except Exception as exception:
        context['error-code'] = exception.__class__.__name__
        _logger.error()

    return None

if __name__ == "__main__":

    context = {}

    if len(sys.argv) > 0:

        try:

            context = json.loads(b64decode(sys.argv[2]))

            winrm_client = get_winrm_client(context,context['timeout'])

            if winrm_client is not None and winrm_client.connected:
                result = discover(winrm_client,context)

                if result is not None:
                    context['result'] = result
                else:
                    if context.get('error-code') is None:
                        if winrm_client.exception is not None:
                            context['error-code'] = winrm_client.exception.__class__.__name__
                        else:
                            context['error-code'] = "Failed to discover"
            else:
                if context.get('error-code') is None:
                    if winrm_client.exception is not None:
                        context['error-code'] = winrm_client.exception.__class__.__name__
                    else:
                        context['error-code'] = "Failed to discover"
            winrm_client.destroy()

            del context['password']

            del context['username']

        except Exception as exception:
            context['error-code'] = exception.__class__.__name__
            _logger.error()
        finally:
            print (json.dumps(context))